//(c) Loe Feijs for Digital Craftmanship.
//Each lissajous figure is approximated by straight line segments
// We work from our own equations: 
// x = (a/2) * cos(N * i * dphi);
// y = (b/2) * sin(M * i * dphi);
//The lissajous figures are generated at random positions
//in a generate-and-test approach. There are two test criteria:
// 1. no overlap with an existing lissajous figure
// 2. being in a non-red area of the loaded image file (eg kusje.jpg).

import processing.pdf.*;

int XSIZE=int(500);
int YSIZE=int(500);

int N = 2;
int M = 4;

PImage img;

final int MAXL = 1000;  //number of lissajous objects
int     lissajousN = 0; //how many
float[] lissajousX = new float[MAXL];
float[] lissajousY = new float[MAXL];
float[] lissajousR = new float[MAXL];

void memorize(float x, float y, float r){
     //store another lissajous figure in a memory bank
     lissajousX[lissajousN] = x;
     lissajousY[lissajousN] = y;
     lissajousR[lissajousN] = r;
     lissajousN++;
}
boolean overlap(float x,float y,float r){
        //test whether new lissajous x,y,r has overlap
        //comparing to all memorized lissajous figures
        boolean aha = false;
        for (int i=0; i<lissajousN; i++){
             if (dist(x,y,lissajousX[i],lissajousY[i]) < r+lissajousR[i])
                 aha = true;
        }
        return aha;
}

void setup() {
  smooth();
  stroke(0);
  //background(255);
  img = loadImage("kusje.jpg");
  size(XSIZE, YSIZE);
  beginRecord(PDF, "LISSAJOUS_" + str(N) + "_" + str(M) +".pdf");
}

void lissajous(float xc, float yc, float size){
     //xc,yc coordinates of centre
     int steps = 2*(10 + 6*M*N);
     float a = size; //diameter horizontally
     float b = size; //diameter vertically
     float dphi = 2*PI/steps;
     noFill();
     strokeWeight(.5);
     stroke(255,0,0);
     beginShape();
     vertex(xc + a/2 , yc);
     for (int i=0; i<steps + 2; i++){
          float x = xc  + a/2 * cos(N * i * dphi);
          float y = yc  + b/2 * sin(M * i * dphi);
          vertex(x,y);
     }
     endShape();
}

void draw() {
  //image(img,0,0,width,height);
  
  int x = int(random(width));
  int y = int(random(height));

  if (green(img.get(x,y)) < 220 && overlap(x,y,25/2) == false){
      //white areas have high green values, red areas don't
      lissajous(x,y,25);
      memorize(x,y,25/2);
  }
  delay(10);
}

void keyPressed() {
  pushMatrix();
  endRecord();
  exit();
}

